<br />

[![Issues](https://img.shields.io/github/issues/arthurfiorette/tinylibs?logo=github&label=Issues)](https://github.com/arthurfiorette/tinylibs/issues)
[![Stars](https://img.shields.io/github/stars/arthurfiorette/tinylibs?logo=github&label=Stars)](https://github.com/arthurfiorette/tinylibs/stargazers)
[![License](https://img.shields.io/github/license/arthurfiorette/tinylibs?logo=githu&label=License)](https://github.com/arthurfiorette/tinylibs/blob/main/LICENSE)
[![Codecov](https://codecov.io/gh/arthurfiorette/tinylibs/branch/main/graph/badge.svg?token=ML0KGCU0VM)](https://codecov.io/gh/arthurfiorette/tinylibs)
[![FOSSA Status](https://app.fossa.com/api/projects/git%2Bgithub.com%2Farthurfiorette%2Ftinylibs.svg?type=shield)](https://app.fossa.com/projects/git%2Bgithub.com%2Farthurfiorette%2Ftinylibs?ref=badge_shield)
[![Join the chat at https://gitter.im/tinylibs-js-org/community](https://badges.gitter.im/tinylibs-js-org/community.svg)](https://gitter.im/tinylibs-js-org/community?utm_source=badge&utm_medium=badge&utm_campaign=pr-badge&utm_content=badge)
[![Speed Blazing](https://img.shields.io/badge/speed-blazing%20%F0%9F%94%A5-brightgreen.svg)](https://twitter.com/acdlite/status/974390255393505280)

[![Latest Version](https://img.shields.io/npm/v/prisma-typegraphql-pagination)](https://www.npmjs.com/package/prisma-typegraphql-pagination)
[![Downloads](https://img.shields.io/npm/dw/prisma-typegraphql-pagination)](https://www.npmjs.com/package/prisma-typegraphql-pagination)
[![JsDelivr](https://data.jsdelivr.com/v1/package/npm/prisma-typegraphql-pagination/badge?style=rounded)](https://www.jsdelivr.com/package/npm/prisma-typegraphql-pagination)
[![Bundlephobia](https://img.shields.io/bundlephobia/minzip/prisma-typegraphql-pagination/latest?style=flat)](https://bundlephobia.com/package/prisma-typegraphql-pagination@latest)
[![Packagephobia](https://packagephobia.com/badge?p=prisma-typegraphql-pagination@latest)](https://packagephobia.com/result?p=prisma-typegraphql-pagination@latest)

<br />

<div align="center">
  <pre>
  <h1>🧬
Prisma TypeGraphql Pagination</h1>
  </pre>
  <br />
</div>

<h3 align="center">
  <code>Prisma TypeGraphql Pagination</code> builds prisma pagination types for type-graphql.
  <br />
  <br />
</h3>

<br />

```ts
import { createPaginator } from 'prisma-typegraphql-pagination';

const UserPagination = createPaginator(User);

@Resolver(() => User)
export class UserResolver {
  // ...
  async friends(
    @Ctx() { prisma }: GqlContext,
    @Args(() => UserPagination) pagination: Prisma.UserFindManyArgs
  ): Promise<User[]> {
    return prisma.user.findMany({ where: { id }, ...pagination });
    // Or for a field resolver
    return prisma.user.findUnique({ where: { id } }).friends(pagination);
  }
}
```

<br />

## Table of Contents

- [Table of Contents](#table-of-contents)
- [Installing](#installing)
  - [Configuration](#configuration)
  - [Class Validator](#class-validator)
  - [Getting a generated class at runtime.](#getting-a-generated-class-at-runtime)
- [License](#license)

<br />

## Installing

```sh
npm   install prisma-typegraphql-pagination
yarn  add     prisma-typegraphql-pagination
```

```ts
import { createPaginator } from 'prisma-typegraphql-pagination';
import { createAxiosHooks } from 'https://cdn.skypack.dev/prisma-typegraphql-pagination@latest';

// This UserPagination class can be used for any pagination for a User[] result.
// It is the type that type-graphql will understand
const UserPagination = createPaginator(User);

function userPagination(@Args(() => UserPagination) pag: Paginator<User>) {
  // ...
}
```

<br />

### Configuration

This package is entirely built in mind of using TSDocs, and as the possible configuration
is very simple, you can see it in the following file.
**[`src/options.ts`](src/options.ts)**.

<br />

### Class Validator

This package provides an optional class validator integration, it is activated if the
package `class-validator` is installed.

It automatically generates `@Validation` decorators for the pagination types, so you can
expect a more robust and secure pagination.

<br />

### Getting a generated class at runtime.

If, for some reason, you need one of the generated classes, like the `UserFieldsEnum`
class generated by your `createPagination(User)` call, you can get it in the same way as
any other class from `type-graphql`.

After getting the array of all classes, you can do a for-of loop and get the one you need
by checking its name.

```ts
const metadata = getMetadataStorage(); // from type-graphql

metadata.objectTypes; // or
metadata.inputTypes; // or
metadata.enums;

// Example:
for (const type of metadata.objectTypes) {
  if (type.name === 'UserEntityPaginator') {
    // ...
  }
}
```

<br />

## License

Licensed under the **MIT**. See [`LICENSE`](LICENSE) for more informations.

[![FOSSA Status](https://app.fossa.com/api/projects/git%2Bgithub.com%2Farthurfiorette%2Ftinylibs.svg?type=small)](https://app.fossa.com/projects/git%2Bgithub.com%2Farthurfiorette%2Ftinylibs?ref=badge_small)

<br />
